# -*- coding: utf-8

"""
@File       :   spearmanrp.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for calculating the similarity based on Spearman Correlation Coefficient between two CTRDMs '

import numpy as np
from neurora.stuff import fisherz_rdm
from scipy.stats import spearmanr


' a function for calculating the similarity based on Spearman Correlation Coefficient between two CTRDMs '

def spearmanrp_cal(CTRDM1, CTRDM2):
    """
    Calculate the similarity based on Spearman Correlation Coefficient between two CTRDMs

    Parameters
    ----------
    CTRDM1 : array [n_conditions, n_conditions]
        The Cross-Temporal RDM 1.
    CTRDM2 : array [n_conditions, n_conditions]
        The Cross-Temporal RDM 2.

    Returns
    -------
    rp : float, float
        Spearman's correlation coefficient
        A r-value and a p-value.
    """

    # get number of conditions
    n_cons = np.shape(CTRDM1)[0]

    # calculate the number of value above the diagonal in RDM
    n = n_cons * (n_cons - 1)

    # initialize two vectors to store the values above the diagnal of two RDMs
    v1 = np.zeros([n], dtype=np.float64)
    v2 = np.zeros([n], dtype=np.float64)

    # assignment
    nn = 0
    for i in range(n_cons):
        for j in range(n_cons):
            if i != j:
                v1[nn] = CTRDM1[i, j]
                v2[nn] = CTRDM2[i, j]
                nn = nn + 1

    # calculate the Spearman Correlation
    rp = np.array(spearmanr(v1, v2))

    return rp

#r1 = np.random.rand(6, 6)
#r2 = np.random.rand(6, 6)
#a = spearmanrp_cal(r1, r2, fisherz=True)
#print(a)


import zipfile

#filename = 'BaeLuck_2018jn_data.zip'
#data_dir = '/Users/zitonglu/Downloads/'
#filepath = data_dir + filename

def unzipfile(filepath, data_dir):

    with zipfile.ZipFile(filepath, 'r') as zip:
        zip.extractall(data_dir)
    print("Unzip completes!")

# test
#unzipfile(filepath, data_dir)


# -*- coding: utf-8

"""
@File       :   ctsimilarities_plot.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a modulate of plotting the Cross-Temporal Similarities '

import numpy as np
import matplotlib.pyplot as plt


' a function for plotting the Cross-Temporal Similarities averaging all subjects'

def ctsimilarities_plot(CTSimilarities, start_time=0, end_time=1, cmap='bwr', lim=[-0.1, 0.1]):

    """
    Plot the Cross-Temporal Similarities averaging all subjects

    Parameters
    ----------
    CTSimilarities : array
        The Cross-Temporal Similarities.
        The size of CTSimilarities should be [n_subs, n_ts, n_ts] or [n_subs, n_ts, n_ts, 2]. n_subs, n_ts represent the
        number of subjects and number of time-points. 2 represents the similarity and a p-value.
    start_time : int or float. Default is 0.
        The start time.
    end_time : int or float. Default is 1.
        The end time.
    cmap : matplotlib colormap or None. Default is 'bwr'.
        The colormap for the figure.
    lim : array or list [min, max]. Default is [-0.1, 0.1].
        The similarity view lims.
    """

    n = len(np.shape(CTSimilarities))

    if n == 4:
        CTSimilarities = CTSimilarities[:, :, :, 0]

    avg_sim = np.average(CTSimilarities, axis=0)

    ax = plt.gca()
    ax.spines["top"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_linewidth(2)
    ax.spines["bottom"].set_linewidth(2)

    minlim = lim[0]
    maxlim = lim[1]

    plt.imshow(avg_sim, extent=(start_time, end_time, start_time, end_time), origin='low', cmap=cmap,
               clim=(minlim, maxlim))

    cb = plt.colorbar(ticks=[minlim, maxlim])
    cb.ax.tick_params(labelsize=12)
    font = {'size': 15,}
    cb.set_label('Representational Similarity', fontdict=font)
    plt.yticks(fontsize=12)
    plt.xticks(fontsize=12)
    plt.xlabel("Training Time-point (s)", fontsize=16)
    plt.ylabel("Test Time-point (s)", fontsize=16)
    plt.show()


' a function for plotting the Cross-Temporal Similarities for all subjects'

def ctsimilarities_plot_bysub(CTSimilarities, start_time=0, end_time=1, cmap='bwr', lim=[-0.1, 0.1], sub_ids=None):

    """
    Plot the Cross-Temporal Similarities for all subjects

    Parameters
    ----------
    CTSimilarities : array
        The Cross-Temporal Similarities.
        The size of CTSimilarities should be [n_subs, n_ts, n_ts] or [n_subs, n_ts, n_ts, 2]. n_subs, n_ts represent the
        number of subjects and number of time-points. 2 represents the similarity and a p-value.
    start_time : int or float. Default is 0.
        The start time.
    end_time : int or float. Default is 1.
        The end time.
    cmap : matplotlib colormap or None. Default is 'bwr'.
        The colormap for the figure.
    lim : array or list [min, max]. Default is [-0.1, 0.1].
        The similarity view lims.
    sub_ids : string-array or string-list. Default is None.
        The subject IDs.
    """

    n = len(np.shape(CTSimilarities))

    if n == 4:
        CTSimilarities = CTSimilarities[:, :, :, 0]

    nsubs = np.shape(CTSimilarities)[0]

    for sub in range(nsubs):

        subsim = CTSimilarities[sub]

        ax = plt.gca()
        ax.spines["top"].set_visible(False)
        ax.spines["right"].set_visible(False)
        ax.spines["left"].set_linewidth(2)
        ax.spines["bottom"].set_linewidth(2)

        minlim = lim[0]
        maxlim = lim[1]

        plt.imshow(subsim, extent=(start_time, end_time, start_time, end_time), origin='low', cmap=cmap,
                    clim=(minlim, maxlim))

        cb = plt.colorbar(ticks=[minlim, maxlim])
        cb.ax.tick_params(labelsize=12)
        font = {'size': 15,}
        cb.set_label('Representational Similarity', fontdict=font)
        plt.yticks(fontsize=12)
        plt.xticks(fontsize=12)
        plt.xlabel("Training Time-point (s)", fontsize=16)
        plt.ylabel("Test Time-point (s)", fontsize=16)
        if sub_ids != None:
            plt.title("Subject "+sub_ids[sub], fontsize=18)
        else:
            plt.title("Subject "+str(sub+1), fontsize=18)
        plt.show()


' a function for plotting the Cross-Temporal Similarities for all channels'

def ctsimilarities_plot_bysub(CTSimilarities, start_time=0, end_time=1, cmap='bwr', lim=[-0.1, 0.1], chl_ids=None):

    """
    Plot the Cross-Temporal Similarities for all channels

    Parameters
    ----------
    CTSimilarities : array
        The Cross-Temporal Similarities.
        The size of CTSimilarities should be [n_channels, n_ts, n_ts] or [n_channels, n_ts, n_ts, 2]. n_channels, n_ts
        represent the number of channels and number of time-points. 2 represents the similarity and a p-value.
    start_time : int or float. Default is 0.
        The start time.
    end_time : int or float. Default is 1.
        The end time.
    cmap : matplotlib colormap or None. Default is 'bwr'.
        The colormap for the figure.
    lim : array or list [min, max]. Default is [-0.1, 0.1].
        The similarity view lims.
    chl_ids : string-array or string-list. Default is None.
        The channel IDs.
    """

    n = len(np.shape(CTSimilarities))

    if n == 4:
        CTSimilarities = CTSimilarities[:, :, :, 0]

    nchls = np.shape(CTSimilarities)[0]

    for chl in range(nchls):

        chlsim = CTSimilarities[chl]

        ax = plt.gca()
        ax.spines["top"].set_visible(False)
        ax.spines["right"].set_visible(False)
        ax.spines["left"].set_linewidth(2)
        ax.spines["bottom"].set_linewidth(2)

        minlim = lim[0]
        maxlim = lim[1]

        plt.imshow(chlsim, extent=(start_time, end_time, start_time, end_time), origin='low', cmap=cmap,
                    clim=(minlim, maxlim))

        cb = plt.colorbar(ticks=[minlim, maxlim])
        cb.ax.tick_params(labelsize=12)
        font = {'size': 15,}
        cb.set_label('Representational Similarity', fontdict=font)
        plt.yticks(fontsize=12)
        plt.xticks(fontsize=12)
        plt.xlabel("Training Time-point (s)", fontsize=16)
        plt.ylabel("Test Time-point (s)", fontsize=16)
        if chl_ids != None:
            plt.title("Channel " + chl_ids[chl], fontsize=18)
        else:
            plt.title("Channel " + str(chl + 1), fontsize=18)
        plt.show()


' a function for plotting the Cross-Temporal Similarities for all subjects and channels'

def ctsimilarities_plot_bysub(CTSimilarities, start_time=0, end_time=1, cmap='bwr', lim=[-0.1, 0.1], sub_ids=None, chl_ids=None):

    """
    Plot the Cross-Temporal Similarities for all subjects and channels

    Parameters
    ----------
    CTSimilarities : array
        The Cross-Temporal Similarities.
        The size of CTSimilarities should be [n_subs, n_channels, n_ts, n_ts] or [n_subs, n_channels, n_ts, n_ts, 2].
        n_subs, n_channels, n_ts represent the number of subjects, the number of channels and number of time-points.
        2 represents the similarity and a p-value.
    start_time : int or float. Default is 0.
        The start time.
    end_time : int or float. Default is 1.
        The end time.
    cmap : matplotlib colormap or None. Default is 'bwr'.
        The colormap for the figure.
    lim : array or list [min, max]. Default is [-0.1, 0.1].
        The similarity view lims.
    sub_ids : string-array or string-list. Default is None.
        The subject IDs.
    chl_ids : string-array or string-list. Default is None.
        The channel IDs.
    """

    n = len(np.shape(CTSimilarities))

    if n == 5:
        CTSimilarities = CTSimilarities[:, :, :, :, 0]

    nsubs = np.shape(CTSimilarities)[0]
    nchls = np.shape(CTSimilarities)[1]

    for sub in range(nsubs):
        for chl in range(nchls):

            subchlsim = CTSimilarities[sub, chl]

            ax = plt.gca()
            ax.spines["top"].set_visible(False)
            ax.spines["right"].set_visible(False)
            ax.spines["left"].set_linewidth(2)
            ax.spines["bottom"].set_linewidth(2)

            minlim = lim[0]
            maxlim = lim[1]

            plt.imshow(subchlsim, extent=(start_time, end_time, start_time, end_time), origin='low', cmap=cmap,
                        clim=(minlim, maxlim))

            cb = plt.colorbar(ticks=[minlim, maxlim])
            cb.ax.tick_params(labelsize=12)
            font = {'size': 15,}
            cb.set_label('Representational Similarity', fontdict=font)
            plt.yticks(fontsize=12)
            plt.xticks(fontsize=12)
            plt.xlabel("Training Time-point (s)", fontsize=16)
            plt.ylabel("Test Time-point (s)", fontsize=16)
            if sub_ids != None and chl_ids != None:
                plt.title("Subject" + sub_ids[sub] + "Channel " + chl_ids[chl], fontsize=18)
            elif sub_ids != None and chl_ids == None:
                plt.title("Subject" + sub_ids[sub] + "Channel " + str(chl+1), fontsize=18)
            elif sub_ids == None and chl_ids != None:
                plt.title("Subject" + str(sub+1) + "Channel " + chl_ids[chl], fontsize=18)
            else:
                plt.title("Subject" + str(sub+1) + "Channel " + str(chl+1), fontsize=18)
            plt.show()





# -*- coding: utf-8

"""
@File       :   fitctrdm.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for calculating Cross-Temporal Similarities between CTRDMs and a Coding Model RDM '

import numpy as np
from pyctrsa.util.progressbar import show_progressbar
from pyctrsa.similarity import spearmanrp, pearsonrp, kendallrp, cosinesimilarity, euclideandistance


' a function for calculating Cross-Temporal Similarities between CTRDMs and a Coding Model RDM '

def ctsimilarities_cal(CTRDMs, Model_RDM, method='spearman'):

    """
    Calculate the Cross-Temporal Similarities between CTRDMs and a Coding Model RDM

    Parameters
    ----------
    CTRDMs : array
        The Cross-Temporal Representational Dissimilarity Matrices.
        The shape could be [n_ts, n_ts, n_conditions, n_conditions] or [n_subs, n_ts, n_ts, n_conditions, n_conditions]
        or [n_channels, n_ts, n_ts, n_conditions, n_conditionss] or [n_subs, n_channels, n_ts, n_ts, n_conditions,
        n_conditions]. n_ts, n_conditions, n_subs, n_channels represent the number of time-points, the number of
        conditions, the number of subjects and the number of channels, respectively.
    Model_RDM : array [n_conditions, n_conditions].
        The Coding Model RDM.
    method : string 'spearman' or 'pearson' or 'kendall' or 'similarity' or 'distance'. Default is 'spearman'.
        The method to calculate the similarities.
        If method='spearman', calculate the Spearman Correlations. If method='pearson', calculate the Pearson
        Correlations. If methd='kendall', calculate the Kendall tau Correlations. If method='similarity', calculate the
        Cosine Similarities. If method='distance', calculate the Euclidean Distances.

    Returns
    -------
    CTSimilarities : array
        Cross-temporal similarities.
        If method='spearman' or 'pearson' or 'kendall':
            If the shape of CTRDMs is [n_ts, n_ts, n_conditions, n_conditions], the shape of CTSimilarities will be
            [n_ts, n_ts, 2].
            If the shape of CTRDMs is [n_subs, n_ts, n_ts, n_conditions, n_conditions], the shape of CTSimilarities will
            be [n_subs, n_ts, n_ts, 2].
            If the shape of CTRDMs is [n_channels, n_ts, n_ts, n_conditions, n_conditions], the shape of CTSimilarities
            will be [n_channels, n_ts, n_ts, 2].
            If the shape of CTRDMs is [n_subs, n_channels, n_ts, n_ts, n_conditions, n_conditions], the shape of
            CTSimilarities will be [n_subs, n_channels, n_ts, n_ts, 2].
        If method='similarity' or 'distance':
            If the shape of CTRDMs is [n_ts, n_ts, n_conditions, n_conditions], the shape of CTSimilarities will be
            [n_ts, n_ts].
            If the shape of CTRDMs is [n_subs, n_ts, n_ts, n_conditions, n_conditions], the shape of CTSimilarities will
            be [n_subs, n_ts, n_ts].
            If the shape of CTRDMs is [n_channels, n_ts, n_ts, n_conditions, n_conditions], the shape of CTSimilarities
            will be [n_channels, n_ts, n_ts].
            If the shape of CTRDMs is [n_subs, n_channels, n_ts, n_ts, n_conditions, n_conditions], the shape of
            CTSimilarities will be [n_subs, n_channels, n_ts, n_ts].

    Notes
    -----
    Users can calculate CTRDMs by pyctrsa.ctrdm.single_cal module and pyctrsa.ctrdm.nulti_cal module
    (zitonglu1996.github.io/pyctrsa/)
    """

    n = len(np.shape(CTRDMs))

    if n == 4:

        n_ts, n_cons = np.shape(CTRDMs)[1:3]

        CTSimilarities = np.zeros([n_ts, n_ts, 2], dtype=np.float)

        total = n_ts * n_ts

        for t1 in range(n_ts):
            for t2 in range(n_ts):

                percent = (t1 * n_ts + t2) / total * 100
                show_progressbar("Calculating", percent)

                if method == 'spearman':
                    CTSimilarities[t1, t2] = spearmanrp.spearmanrp_cal(CTRDMs[t1, t2], Model_RDM)
                if method == 'pearson':
                    CTSimilarities[t1, t2] = pearsonrp.pearsonrp_cal(CTRDMs[t1, t2], Model_RDM)
                if method == 'kendall':
                    CTSimilarities[t1, t2] = kendallrp.kendallrp_cal(CTRDMs[t1, t2], Model_RDM)
                if method == 'similarity':
                    CTSimilarities[t1, t2, 0] = cosinesimilarity.cosinesimilarity_cal(CTRDMs[t1, t2], Model_RDM)
                if method == 'distance':
                    CTSimilarities[t1, t2, 0] = euclideandistance.euclideandistance_cal(CTRDMs[t1, t2], Model_RDM)

        if method == 'spearman' or method == 'pearson' or method == 'kendall':

            return CTSimilarities

        if method == 'similarity' or method == 'distance':

            return CTSimilarities[:, :, 0]

    if n == 5:

        n1 = np.shape(CTRDMs)[0]
        n_ts, n_cons = np.shape(CTRDMs)[2:4]

        CTSimilarities = np.zeros([n1, n_ts, n_ts, 2], dtype=np.float)

        total = n1 * n_ts * n_ts

        for i in range(n1):
            for t1 in range(n_ts):
                for t2 in range(n_ts):

                    percent = (i * n_ts * n_ts + t1 * n_ts + t2) / total * 100
                    show_progressbar("Calculating", percent)

                    if method == 'spearman':
                        CTSimilarities[i, t1, t2] = spearmanrp.spearmanrp_cal(CTRDMs[i, t1, t2], Model_RDM)
                        #print(CTSimilarities[i, t1, t2])
                    if method == 'pearson':
                        CTSimilarities[i, t1, t2] = pearsonrp.pearsonrp_cal(CTRDMs[i, t1, t2], Model_RDM)
                    if method == 'kendall':
                        CTSimilarities[i, t1, t2] = kendallrp.kendallrp_cal(CTRDMs[i, t1, t2], Model_RDM)
                    if method == 'similarity':
                        CTSimilarities[i, t1, t2, 0] = cosinesimilarity.cosinesimilarity_cal(CTRDMs[i, t1, t2], Model_RDM)
                    if method == 'distance':
                        CTSimilarities[i, t1, t2, 0] = euclideandistance.euclideandistance_cal(CTRDMs[i, t1, t2], Model_RDM)

        if method == 'spearman' or method == 'pearson' or method == 'kendall':
            return CTSimilarities

        if method == 'similarity' or method == 'distance':
            return CTSimilarities[:, :, :, 0]

    if n == 6:

        n1, n2 = np.shape(CTRDMs)[:2]
        n_ts, n_cons = np.shape(CTRDMs)[3:5]

        CTSimilarities = np.zeros([n1, n2, n_ts, n_ts, 2], dtype=np.float)

        total = n1 * n2 * n_ts * n_ts

        for i in range(n1):
            for j in range(n2):
                for t1 in range(n_ts):
                    for t2 in range(n_ts):

                        percent = (i * n2 * n_ts * n_ts + j * n_ts * n_ts + t1 * n_ts + t2) / total * 100
                        show_progressbar("Calculating", percent)

                        if method == 'spearman':
                            CTSimilarities[i, j, t1, t2] = spearmanrp.spearmanrp_cal(CTRDMs[i, j, t1, t2], Model_RDM)
                        if method == 'pearson':
                            CTSimilarities[i, j, t1, t2] = pearsonrp.pearsonrp_cal(CTRDMs[i, j, t1, t2], Model_RDM)
                        if method == 'kendall':
                            CTSimilarities[i, j, t1, t2] = kendallrp.kendallrp_cal(CTRDMs[i, j, t1, t2], Model_RDM)
                        if method == 'similarity':
                            CTSimilarities[i, j, t1, t2, 0] = cosinesimilarity.cosinesimilarity_cal(CTRDMs[i, j, t1, t2], Model_RDM)
                        if method == 'distance':
                            CTSimilarities[i, j, t1, t2, 0] = euclideandistance.euclideandistance_cal(CTRDMs[i, j, t1, t2], Model_RDM)

        if method == 'spearman' or method == 'pearson' or method == 'kendall':
            return CTSimilarities

        if method == 'similarity' or method == 'distance':
            return CTSimilarities[:, :, :, :, 0]

# test codes
#a = np.random.rand(100, 100, 6, 6)
#c = np.random.rand(6, 6)
#b = ctsimilarities_cal(a, c)
#print(b)


# -*- coding: utf-8

"""
@File       :   preprocess_data.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

import numpy as np
import os
import scipy.io as sio
import h5py
from pyctrsa.util.progressbar import show_progressbar

def pre_data(subs, data_dir):

    newdata_dir = data_dir + 'data_for_CTRSA/'

    if os.path.exists(newdata_dir) == False:
        os.makedirs(newdata_dir)
    if os.path.exists(newdata_dir + 'ERP/') == False:
        os.makedirs(newdata_dir + 'ERP/')
    if os.path.exists(newdata_dir + 'Alpha/') == False:
        os.makedirs(newdata_dir + 'Alpha/')

    n = len(subs)
    subindex = 0

    for sub in subs:
        data = sio.loadmat(data_dir + "data/ERP" + sub + ".mat")["filtData"][:, :, 250:]
        # data.shape: n_trials, n_channels, n_times

        ori_label = np.loadtxt(data_dir + "labels/ori_" + sub + ".txt")[:, 1]
        pos_label = np.loadtxt(data_dir + "labels/pos_" + sub + ".txt")[:, 1]

        ori_subdata = np.zeros([16, 40, 27, 500], dtype=np.float)
        pos_subdata = np.zeros([16, 40, 27, 500], dtype=np.float)

        ori_labelindex = np.zeros([16], dtype=np.int)
        pos_labelindex = np.zeros([16], dtype=np.int)

        for i in range(640):
            label = int(ori_label[i])
            ori_subdata[label, ori_labelindex[label]] = data[i]
            ori_labelindex[label] = ori_labelindex[label] + 1
            label = int(pos_label[i])
            pos_subdata[label, pos_labelindex[label]] = data[i]
            pos_labelindex[label] = pos_labelindex[label] + 1

        f = h5py.File(newdata_dir + "ERP/" + sub + ".h5", "w")
        f.create_dataset("ori", data=ori_subdata)
        f.create_dataset("pos", data=pos_subdata)
        f.close()

        data = sio.loadmat(data_dir + "data/Alpha" + sub + ".mat")["filtData"][:, :, 250:]
        # data.shape: n_trials, n_channels, n_times

        ori_label = np.loadtxt(data_dir + "labels/ori_" + sub + ".txt")[:, 1]
        pos_label = np.loadtxt(data_dir + "labels/pos_" + sub + ".txt")[:, 1]

        ori_subdata = np.zeros([16, 40, 27, 500], dtype=np.float)
        pos_subdata = np.zeros([16, 40, 27, 500], dtype=np.float)

        ori_labelindex = np.zeros([16], dtype=np.int)
        pos_labelindex = np.zeros([16], dtype=np.int)

        for i in range(640):
            label = int(ori_label[i])
            ori_subdata[label, ori_labelindex[label]] = data[i]
            ori_labelindex[label] = ori_labelindex[label] + 1
            label = int(pos_label[i])
            pos_subdata[label, pos_labelindex[label]] = data[i]
            pos_labelindex[label] = pos_labelindex[label] + 1

        subindex = subindex + 1
        percent = subindex/n*100
        show_progressbar("Preprocessing", percent)

        f = h5py.File(newdata_dir + "Alpha/" + sub + ".h5", "w")
        f.create_dataset("ori", data=ori_subdata)
        f.create_dataset("pos", data=pos_subdata)
        f.close()



# -*- coding: utf-8

"""
@File       :   multi_cal.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for calculating Cross-Temporal RDMs for multi-channels data '

import numpy as np
from pyctrsa.util.progressbar import show_progressbar
from pyctrsa.ctrdm import single_cal
from scipy.stats import pearsonr

np.seterr(divide='ignore', invalid='ignore')


' A function to calculate Cross-Temporal RDMs for multi-channels '

def ctrdms_cal(data, sub_opt=1, chl_opt=1, time_win=10, time_step=5):

    """
    a function to calculate CTRDMs for multi-channels

    Parameters
    ----------
    data : array
        EEG/MEG data from a time-window.
        The shape of data should be [n_conditions, n_subs, n_channels, n_ts]. n_conditions, n_subs, n_channels, n_ts
        represent the number of conditions, the number of subjects, the number of channels and the number of time-points
        respectively.
    sub_opt : int 0 or 1. Default is 1.
        Caculate the CTRDMs for each subject or not.
        If sub_opt=1, return the CTRDMs for each subjects.
        If sub_opt=0, return the avg CTRDMs among all subjects.
    chl_opt : int 0 or 1. Default is 1.
        Caculate the CTRDMs for each channel or not.
        If chl_opt=1, calculate the CTRDMs for each channel.
        If chl_opt=0, calculate the CTRDMs after averaging the channels.
    time_win : int. Default is 10.
        Set a time-window for calculating the CTRDM for different time-points.
        If time_win=10, that means each calculation process based on 10 time-points.
    time_step : int. Default is 5.
        The time step size for each time of calculating.

    Returns
    -------
    CTRDMs : array [int((n_ts-time_win)/time_step)+1, int((n_ts-time_win)/time_step)+1, n_conditions, n_conditions]
        Cross-Temporal RDMs.
    """

    n_cons, n_subs, n_chls, n_ts = np.shape(data)

    nts = int((n_ts - time_win) / time_step) + 1

    # chl_opt=0
    if chl_opt == 0:

        data_for_cal = np.zeros([n_cons, n_subs, nts, n_chls, time_win], dtype=np.float)

        for con in range(n_cons):
            for sub in range(n_subs):
                for t in range(nts):
                    for chl in range(n_chls):
                        data_for_cal[con, sub, t, chl] = data[con, sub, chl, t * time_step:t * time_step + time_win]

        data_for_cal = np.reshape(data_for_cal, [n_cons, n_subs, nts, n_chls*time_win])

        ctrdms = np.zeros([n_subs, nts, nts, n_cons, n_cons], dtype=np.float)

        total = n_subs * nts * nts

        for sub in range(n_subs):
            for t1 in range(nts):
                for t2 in range(nts):
                    percent = (sub * nts * nts + t1 * nts + t2) / total * 100
                    show_progressbar("Calculating", percent)

                    for con1 in range(n_cons):
                        for con2 in range(n_cons):

                            if con1 != con2:
                                r = pearsonr(data_for_cal[con1, sub, t1], data_for_cal[con2, sub, t2])[0]
                                ctrdms[sub, t1, t2, con1, con2] = 1 - r
                            if con1 == con2:
                                ctrdms[sub, t1, t2, con1, con2] = 0

        # chl_opt=0 & sub_opt=0
        if sub_opt == 0:

            return np.average(ctrdms, axis=0)

        # chl_opt=0 & sub_opt=1
        else:

            return ctrdms

    # chl_opt=1
    else:

        ctrdms = np.zeros([n_subs, n_chls, nts, nts, n_cons, n_cons], dtype=np.float)

        for sub in range(n_subs):
            for chl in range(n_chls):

                print("\nSubject "+str(sub+1)+" Channel "+str(chl+1)+":")

                ctrdms[sub, chl] = single_cal.ctrdm_cal(data[:, sub, chl], time_win=time_win, time_step=time_step)

        # chl_opt=1 & sub_opt=0
        if sub_opt == 0:

            return np.average(ctrdms, axis=0)

        # chl_opt=1 & sub_opt=1
        else:

            return ctrdms

# test
#data = np.random.rand(16, 40, 5, 50)
#from pyctrsa.plotting import ctrdm
#ctrdms = ctrdms_cal(data, sub_opt=1, chl_opt=0)
#ctrdms = np.average(ctrdms, axis=0)
#ctrdm.ctrdm_plot(ctrdms[1, 3])


# -*- coding: utf-8

"""
@File       :   euclideandistance.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for calculating the similarity based on Euclidean Distance between two CTRDMs '

import numpy as np


' a function for calculating the similarity based on Euclidean Distance between two CTRDMs '

def euclideandistance_cal(CTRDM1, CTRDM2):
    """
    Calculate the similarity based on Euclidean Distance between two CTRDMs

    Parameters
    ----------
    CTRDM1 : array [n_conditions, n_conditions]
        The Cross-Temporal RDM 1.
    CTRDM2 : array [n_conditions, n_conditions]
        The Cross-Temporal RDM 2.

    Returns
    -------
    distance : float
        Euclidean Distance.
    """

    # get number of conditions
    n_cons = np.shape(CTRDM1)[0]

    # calculate the number of value above the diagonal in RDM
    n = n_cons * (n_cons - 1)

    # initialize two vectors to store the values above the diagnal of two RDMs
    v1 = np.zeros([n], dtype=np.float64)
    v2 = np.zeros([n], dtype=np.float64)

    # assignment
    nn = 0
    for i in range(n_cons):
        for j in range(n_cons):
            if i != j:
                v1[nn] = CTRDM1[i, j]
                v2[nn] = CTRDM2[i, j]
                nn = nn + 1

    # calculate the Euclidean Distance
    distance = np.linalg.norm(v1 - v2)

    return distance


# -*- coding: utf-8

"""
@File       :   tbytsimilarities.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for plotting the time-by-time similarities '

import numpy as np
from neurora.stuff import permutation_test
import matplotlib.pyplot as plt
from neurora.rsa_plot import plot_corrs_hotmap


' a function for plotting the time-by-time similarities averaging all subjects '

def tbytsimilarities_plot(CTSimilarities, start_time=0, end_time=1, color='r', lim=[-0.1, 0.8]):

    """
    Plot the time-by-time Similarities averaging all subjects

    Parameters
    ----------
    CTSimilarities : array
        The Cross-Temporal Similarities.
        The size of CTSimilarities should be [n_subs, n_ts, n_ts] or [n_subs, n_ts, n_ts, 2]. n_subs, n_ts represent the
        number of subjects and number of time-points. 2 represents the similarity and a p-value.
    start_time : int or float. Default is 0.
        The start time.
    end_time : int or float. Default is 1.
        The end time.
    color : matplotlib color or None. Default is 'r'.
        The color for the curve.
    lim : array or list [min, max]. Default is [-0.1, 0.8].
        The corrs view lims.
    """

    n = len(np.shape(CTSimilarities))

    if n == 4:
        CTSimilarities = CTSimilarities[:, :, :, 0]

    nsubs = np.shape(CTSimilarities)[0]
    nts = np.shape(CTSimilarities)[1]

    tstep = float((end_time-start_time)/nts)

    sim = np.zeros([nsubs, nts], dtype=np.float)

    for sub in range(nsubs):
        for t in range(nts):

            sim[sub, t] = CTSimilarities[sub, t, t]

    for sub in range(nsubs):
        for t in range(nts):

            if t<=1:
                sim[sub, t] = np.average(sim[sub, :t+3])
            if t>1 and t<(nts-2):
                sim[sub, t] = np.average(sim[sub, t-2:t+3])
            if t>=(nts-2):
                sim[sub, t] = np.average(sim[sub, t-2:])

    avg = np.average(sim, axis=0)
    err = np.zeros([nts], dtype=np.float)

    for t in range(nts):
        err[t] = np.std(sim[:, t], ddof=1)/np.sqrt(nsubs)

    ps = np.zeros([nts], dtype=np.float)
    chance = np.full([nsubs], 0)

    for t in range(nts):
        ps[t] = permutation_test(sim[:, t], chance)
        if ps[t] < 0.05 and avg[t] > 0:
            plt.plot(t*0.5*tstep+start_time, start_time+(end_time-start_time)*0.9, 's', color=color, alpha=1)
            xi = [t*tstep+start_time, t*tstep+tstep+start_time]
            ymin = [0]
            ymax = [avg[t]-err[t]]
            plt.fill_between(xi, ymax, ymin, facecolor=color, alpha=0.1)

    ax = plt.gca()
    ax.spines["top"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_linewidth(3)
    ax.spines["bottom"].set_linewidth(3)
    ax.spines['bottom'].set_position(('data', 0))

    print(start_time, end_time, tstep)
    x = np.arange(start_time+0.5*tstep, end_time+0.5*tstep, tstep)
    plt.fill_between(x, avg + err, avg - err, facecolor="orangered", alpha=0.8)
    minlim = lim[0]
    maxlim = lim[1]
    plt.ylim(minlim, maxlim)
    plt.xlim(start_time, end_time)
    plt.tick_params(labelsize=12)
    plt.xlabel("Time (s)", fontsize=16)
    plt.ylabel("Representational Similarity", fontsize=16)
    plt.show()


' a function for plotting the time-by-time similarities for all subjects '

def tbytsimilarities_plot_bysubject(CTSimilarities, start_time=0, end_time=1, figsize=None, cmap='bwr', lim=[-0.1, 0.8], sub_ids=None):
    """
    Plot the time-by-time Similarities for all subjects

    Parameters
    ----------
    CTSimilarities : array
        The Cross-Temporal Similarities.
        The size of CTSimilarities should be [n_subs, n_ts, n_ts] or [n_subs, n_ts, n_ts, 2]. n_subs, n_ts represent the
        number of subjects and number of time-points. 2 represents the similarity and a p-value.
    start_time : int or float. Default is 0.
        The start time.
    end_time : int or float. Default is 1.
        The end time.
    figsize : array or list, [size_X, size_Y]
        The size of the figure.
        If figsize=None, the size of the figure will be ajusted automatically.
    cmap : matplotlib colormap or None. Default is 'bwr'.
        The colormap for the figure.
    sub_ids : string-array or string-list. Default is None.
        The subject IDs.
    """

    n = len(np.shape(CTSimilarities))

    if n == 4:
        CTSimilarities = CTSimilarities[:, :, :, 0]

    nsubs = np.shape(CTSimilarities)[0]
    nts = np.shape(CTSimilarities)[1]

    tstep = float((end_time - start_time) / nts)

    sim = np.zeros([nsubs, nts], dtype=np.float)

    for sub in range(nsubs):
        for t in range(nts):
            sim[sub, t] = CTSimilarities[sub, t, t]

    for sub in range(nsubs):
        for t in range(nts):

            if t <= 1:
                sim[sub, t] = np.average(sim[sub, :t + 3])
            if t > 1 and t < (nts - 2):
                sim[sub, t] = np.average(sim[sub, t - 2:t + 3])
            if t >= (nts - 2):
                sim[sub, t] = np.average(sim[sub, t - 2:])

    plot_corrs_hotmap(sim, chllabels=sub_ids, time_unit=[start_time, tstep], lim=lim, figsize=figsize, cmap=cmap)


' a function for plotting the time-by-time similarities for all channels '

def tbytsimilarities_plot_bychannel(CTSimilarities, start_time=0, end_time=1, figsize=None, cmap='bwr', lim=[-0.1, 0.8], chl_ids=None):
    """
    Plot the time-by-time Similarities for all channels

    Parameters
    ----------
    CTSimilarities : array
        The Cross-Temporal Similarities.
        The size of CTSimilarities should be [n_subs, n_ts, n_ts] or [n_subs, n_ts, n_ts, 2]. n_subs, n_ts represent the
        number of subjects and number of time-points. 2 represents the similarity and a p-value.
    start_time : int or float. Default is 0.
        The start time.
    end_time : int or float. Default is 1.
        The end time.
    figsize : array or list, [size_X, size_Y]
        The size of the figure.
        If figsize=None, the size of the figure will be ajusted automatically.
    cmap : matplotlib colormap or None. Default is 'bwr'.
        The colormap for the figure.
    chl_ids : string-array or string-list. Default is None.
        The channel IDs.
    """

    n = len(np.shape(CTSimilarities))

    if n == 4:
        CTSimilarities = CTSimilarities[:, :, :, 0]

    nchls = np.shape(CTSimilarities)[0]
    nts = np.shape(CTSimilarities)[1]

    tstep = float((end_time - start_time) / nts)

    sim = np.zeros([nchls, nts], dtype=np.float)

    for chl in range(nchls):
        for t in range(nts):
            sim[chl, t] = CTSimilarities[chl, t, t]

    for chl in range(nchls):
        for t in range(nts):

            if t <= 1:
                sim[chl, t] = np.average(sim[chl, :t + 3])
            if t > 1 and t < (nts - 2):
                sim[chl, t] = np.average(sim[chl, t - 2:t + 3])
            if t >= (nts - 2):
                sim[chl, t] = np.average(sim[chl, t - 2:])

    plot_corrs_hotmap(sim, chllabels=chl_ids, time_unit=[start_time, tstep], lim=lim, figsize=figsize, cmap=cmap)




# -*- coding: utf-8

"""
@File       :   ctrdm_cal.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for calculating Cross-Temporal RDMs for a single channel & a single subject '

import numpy as np
from pyctrsa.util.progressbar import show_progressbar
from scipy.stats import pearsonr

np.seterr(divide='ignore', invalid='ignore')


' A function to calculate Cross-Temporal RDMs for a single channel & a single subject '

def ctrdm_cal(data, time_win=10, time_step=5):

    """
    a function to calculate CTRDMs for a single channel & a single subject

    Parameters
    ----------
    data : array
        EEG/MEG data from a time-window.
        The shape of data should be [n_conditions, n_ts]. n_conditions, n_ts represent the number of conditions and the
        number of time-points respectively.
    time_win : int. Default is 10.
        Set a time-window for calculating the CTRDM for different time-points.
        If time_win=10, that means each calculation process based on 10 time-points.
    time_step : int. Default is 5.
        The time step size for each time of calculating.

    Returns
    -------
    CTRDMs : array [int((n_ts-time_win)/time_step)+1, int((n_ts-time_win)/time_step)+1, n_conditions, n_conditions]
        Cross-Temporal RDMs.
    """

    n_cons, n_ts = np.shape(data)

    nts = int((n_ts - time_win) / time_step) + 1

    data_for_cal = np.zeros([n_cons, nts, time_win], dtype=np.float)

    for con in range(n_cons):
        for t in range(nts):
            data_for_cal[con, t] = data[con, t*time_step:t*time_step+time_win]

    ctrdms = np.zeros([nts, nts, n_cons, n_cons], dtype=np.float)

    total = nts*nts

    for t1 in range(nts):
        for t2 in range(nts):

            percent = (t1*nts + t2)/total*100
            show_progressbar("Calculating", percent)

            for con1 in range(n_cons):
                for con2 in range(n_cons):

                    if con1 != con2:
                        r = pearsonr(data_for_cal[con1, t1], data_for_cal[con2, t2])[0]
                        ctrdms[t1, t2, con1, con2] = 1-r
                    if con1 == con2:
                        ctrdms[t1, t2, con1, con2] = 0

    return ctrdms

# test
#data = np.random.rand(5, 80)
#print(ctrdm_cal(data)[1, 2])

#from pyctrsa.plotting import ctrdm
#ctrdm.ctrdm_plot(ctrdm_cal(data)[1, 2])



# -*- coding: utf-8

"""
@File       :   normalrdmbased.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for calculating Cross-Temporal Similarities based on normal RDMs '

import numpy as np
from neurora.rdm_corr import rdm_correlation_spearman, rdm_correlation_pearson, rdm_correlation_kendall
from neurora.rdm_corr import rdm_similarity, rdm_distance


' a function for calculating Cross-Temporal Similarities based on normal RDMs '

def ctsimilarities_cal(RDMs, method='spearman', fisherz=True):

    """
    Calculate the Cross-Temporal Similarities based on normal RDMs

    Parameters
    ----------
    RDMs : array
        The Representational Dissimilarity Matrices in time series.
        The shape could be [n_ts, n_conditions, n_conditions] or [n_subs, n_ts, n_conditions, n_conditions] or
        [n_channels, n_ts, n_conditions, n_conditions] or [n_subs, n_channels, n_ts, n_conditions, n_conditions].
        n_ts, n_conditions, n_subs, n_channels represent the number of time-points, the number of conditions, the number
        of subjects and the number of channels, respectively.
    method : string 'spearman' or 'pearson' or 'kendall' or 'similarity' or 'distance'. Default is 'spearman'.
        The method to calculate the similarities.
        If method='spearman', calculate the Spearman Correlations. If method='pearson', calculate the Pearson
        Correlations. If methd='kendall', calculate the Kendall tau Correlations. If method='similarity', calculate the
        Cosine Similarities. If method='distance', calculate the Euclidean Distances.
    fisherz : bool True or False. Default is False.
        Do the Fisher-Z transform of the CTRDMs or not.
        Only when method='spearman' or 'pearson' or 'kendall', it works.

    Returns
    -------
    CTSimilarities : array
        Cross-temporal similarities.
        If the shape of RDMs is [n_ts, n_conditions, n_conditions] and method='spearman' or 'pearson' or 'kendall', the
        shape of CTSimilarities will be [n_ts, n_ts, 2].
        If the shape of RDMs is [n_subs, n_ts, n_conditions, n_conditions] and method='spearman' or 'pearson' or
        'kendall', the shape of CTSimilarities will be [n_subs, n_ts, n_ts, 2].
        If the shape of RDMs is [n_channels, n_ts, n_conditions, n_conditions] and method='spearman' or 'pearson' or
        'kendall', the shape of CTSimilarities will be [n_channels, n_ts, n_ts, 2].
        If the shape of RDMs is [n_subs, n_channels, n_ts, n_conditions, n_conditions] and method='spearman' or
        'pearson' or 'kendall', the shape of CTSimilarities will be [n_subs, n_channels, n_ts, n_ts, 2].
        If the shape of RDMs is [n_ts, n_conditions, n_conditions] and method='similarity' or 'distance', the shape of
        CTSimilarities will be [n_ts, n_ts].
        If the shape of RDMs is [n_subs, n_ts, n_conditions, n_conditions] and method='similarity' or 'distance', the
        shape of CTSimilarities will be [n_subs, n_ts, n_ts].
        If the shape of RDMs is [n_channels, n_ts, n_conditions, n_conditions] and method='similarity' or 'distance',
        the shape of CTSimilarities will be [n_channels, n_ts, n_ts].
        If the shape of RDMs is [n_subs, n_channels, n_ts, n_conditions, n_conditions] and method='similarity' or
        'distance', the shape of CTSimilarities will be [n_subs, n_channels, n_ts, n_ts].

    Notes
    -----
    Users can calculate RDMs by NeuroRA (zitonglu1996.github.io/neurora/)
    """

    n = len(np.shape(RDMs))

    if n == 3:

        n_ts, n_cons = np.shape(RDMs)[:2]

        CTSimilarities = np.zeros([n_ts, n_ts, 2], dtype=np.float)

        for t1 in range(n_ts):
            for t2 in range(n_ts):

                if method == 'spearman':
                    CTSimilarities[t1, t2] = rdm_correlation_spearman(RDMs[t1], RDMs[t2], fisherz=fisherz)
                if method == 'pearson':
                    CTSimilarities[t1, t2] = rdm_correlation_pearson(RDMs[t1], RDMs[t2], fisherz=fisherz)
                if method == 'kendall':
                    CTSimilarities[t1, t2] = rdm_correlation_kendall(RDMs[t1], RDMs[t2], fisherz=fisherz)
                if method == 'similarity':
                    CTSimilarities[t1, t2, 0] = rdm_similarity(RDMs[t1], RDMs[t2])
                if method == 'distance':
                    CTSimilarities[t1, t2, 0] = rdm_distance(RDMs[t1], RDMs[t2])

        if method == 'spearman' or method == 'pearson' or method == 'kendall':

            return CTSimilarities

        if method == 'similarity' or method == 'distance':

            return CTSimilarities[:, :, 0]

    if n == 4:

        n1, n_ts, n_cons = np.shape(RDMs)[:3]

        CTSimilarities = np.zeros([n1, n_ts, n_ts, 2], dtype=np.float)

        for i in range(n1):
            for t1 in range(n_ts):
                for t2 in range(n_ts):

                    if method == 'spearman':
                        CTSimilarities[i, t1, t2] = rdm_correlation_spearman(RDMs[i, t1], RDMs[i, t2], fisherz=fisherz)
                    if method == 'pearson':
                        CTSimilarities[i, t1, t2] = rdm_correlation_pearson(RDMs[i, t1], RDMs[i, t2], fisherz=fisherz)
                    if method == 'kendall':
                        CTSimilarities[i, t1, t2] = rdm_correlation_kendall(RDMs[i, t1], RDMs[i, t2], fisherz=fisherz)
                    if method == 'similarity':
                        CTSimilarities[i, t1, t2, 0] = rdm_similarity(RDMs[i, t1], RDMs[i, t2])
                    if method == 'distance':
                        CTSimilarities[i, t1, t2, 0] = rdm_distance(RDMs[i, t1], RDMs[i, t2])

        if method == 'spearman' or method == 'pearson' or method == 'kendall':
            return CTSimilarities

        if method == 'similarity' or method == 'distance':
            return CTSimilarities[:, :, :, 0]

    if n == 5:

        n1, n2, n_ts, n_cons = np.shape(RDMs)[:4]

        CTSimilarities = np.zeros([n1, n2, n_ts, n_ts, 2], dtype=np.float)

        for i in range(n1):
            for j in range(n2):
                for t1 in range(n_ts):
                    for t2 in range(n_ts):

                        if method == 'spearman':
                            CTSimilarities[i, j, t1, t2] = rdm_correlation_spearman(RDMs[i, j, t1], RDMs[i, j, t2],
                                                                                 fisherz=fisherz)
                        if method == 'pearson':
                            CTSimilarities[i, j, t1, t2] = rdm_correlation_pearson(RDMs[i, j, t1], RDMs[i, j, t2],
                                                                                fisherz=fisherz)
                        if method == 'kendall':
                            CTSimilarities[i, j, t1, t2] = rdm_correlation_kendall(RDMs[i, j, t1], RDMs[i, j, t2],
                                                                                fisherz=fisherz)
                        if method == 'similarity':
                            CTSimilarities[i, j, t1, t2, 0] = rdm_similarity(RDMs[i, j, t1], RDMs[i, j, t2])
                        if method == 'distance':
                            CTSimilarities[i, j, t1, t2, 0] = rdm_distance(RDMs[i, j, t1], RDMs[i, j, t2])

        if method == 'spearman' or method == 'pearson' or method == 'kendall':
            return CTSimilarities

        if method == 'similarity' or method == 'distance':
            return CTSimilarities[:, :, :, :, 0]



# -*- coding: utf-8

"""
@File       :   download_data.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

from six.moves import urllib
import sys
from pyctrsa.util.progressbar import show_progressbar

def schedule(blocknum,blocksize,totalsize):

    if totalsize == 0:
        percent = 0
    else:
        percent = blocknum * blocksize / totalsize
    if percent > 1.0:
        percent = 1.0
    percent = percent * 100
    show_progressbar("Downloading", percent)

# test
#url = 'https://attachment.zhaokuangshi.cn/BaeLuck_2018jn_data.zip'
#filename = 'BaeLuck_2018jn_data.zip'
#data_dir = '/Users/zitonglu/Downloads/'
#filepath = data_dir + filename

#urllib.request.urlretrieve(url, filepath, schedule)
#print('Download completes!')




# -*- coding: utf-8

"""
@File       :   progressbar.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

import sys

def show_progressbar(str, cur, total=100):

    percent = '{:.2%}'.format(cur / total)
    sys.stdout.write('\r')
    sys.stdout.write(str + ": [%-100s] %s" % ('=' * int(cur), percent))
    sys.stdout.flush()


from setuptools import setup, find_packages

setup(
    name='pyctrsa',
    version='0.0.7',
    description=(
        'A Python Toolbox for Cross-Temporal Representational Similarity Analysis-based Decoding on E/MEG Data'
    ),
    long_description=open('README.md').read(),
    author='Zitong Lu',
    author_email='zitonglu1996@gmail.com',
    maintainer='Zitong Lu',
    maintainer_email='zitonglu1996@gmail.com',
    license='MIT License',
    packages=find_packages(),
    platforms=["all"],
    url='https://github.com/ZitongLu1996/PyCTRSA',
    install_requires=[
        'numpy',
        'scipy>=1.4',
        'matplotlib',
        'neurora'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
)



# -*- coding: utf-8

"""
@File       :   makedir.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

import os

def mkdir(path):

    path = path.strip()
    path = path.rstrip("/")

    isExists = os.path.exists(path)

    if not isExists:
        os.makedirs(path)

        print(path + ' is created successfully!')
        return True
    else:
        print(path + ' already exists')
        return False

# test codes
#path = '../data/'
#mkdir(path)


# -*- coding: utf-8

"""
@File       :   normaldatabased.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for calculating Cross-Temporal Similarities between neural data under two conditions '

import numpy as np
from scipy.stats import spearmanr, pearsonr, kendalltau


' a function for calculating Cross-Temporal Similarities between neural data under two conditions '

def ctsimilarities_cal(data1, data2, sub_opt=1, chl_opt=1, time_win=10, time_step=5, method='spearman'):

    """
    Calculate the Cross-Temporal Similarities between neural data under two conditions

    Parameters
    ----------
    data1 : array
        EEG/MEG data from a time-window under condition1.
        The shape of data should be [n_subs, n_channels, n_ts]. n_subs, n_channels, n_ts represent the number of
        conditions, the number of subjects, the number of channels and the number of time-points respectively.
    data2 : array
        EEG/MEG data from a time-window under condition2.
        The shape of data should be [n_subs, n_channels, n_ts]. n_subs, n_channels, n_ts represent the number of
        conditions, the number of subjects, the number of channels and the number of time-points respectively.
    sub_opt : int 0 or 1. Default is 1.
        Caculate the CTRDMs for each subject or not.
        If sub_opt=1, return the CTRDMs for each subjects.
        If sub_opt=0, return the avg CTRDMs among all subjects.
    chl_opt : int 0 or 1. Default is 1.
        Caculate the CTRDMs for each channel or not.
        If chl_opt=1, calculate the CTRDMs for each channel.
        If chl_opt=0, calculate the CTRDMs after averaging the channels.
    time_win : int. Default is 10.
        Set a time-window for calculating the CTRDM for different time-points.
        If time_win=10, that means each calculation process based on 10 time-points.
    time_step : int. Default is 5.
        The time step size for each time of calculating.
    method : string 'spearman' or 'pearson' or 'kendall' or 'similarity' or 'distance'. Default is 'spearman'.
        The method to calculate the similarities.
        If method='spearman', calculate the Spearman Correlations. If method='pearson', calculate the Pearson
        Correlations. If methd='kendall', calculate the Kendall tau Correlations. If method='similarity', calculate the
        Cosine Similarities. If method='distance', calculate the Euclidean Distances.

    Returns
    -------
    CTSimilarities : array
        Cross-temporal similarities.
        If method='spearman' or 'pearson' or 'kendall':
            If sub_opt=1 and chl_opt=1, the shape of CTSimilarities will be [n_subs, n_channels,
            int((n_ts-time_win)/time_step)+1, int((n_ts-time_win)/time_step)+1, 2]
            If sub_opt=1 and chl_opt=0, the shape of CTSimilarities will be [n_subs, int((n_ts-time_win)/time_step)+1,
            int((n_ts-time_win)/time_step)+1, 2]
            If sub_opt=0 and chl_opt=1, the shape of CTSimilarities will be [n_channels, int((n_ts-time_win)/time_step)
            +1, int((n_ts-time_win)/time_step)+1, 2]
            If sub_opt=0 and chl_opt=0, the shape of CTSimilarities will be [int((n_ts-time_win)/time_step)+1,
            int((n_ts-time_win)/time_step)+1, 2]
        If method='similarity' or 'distance':
            If sub_opt=1 and chl_opt=1, the shape of CTSimilarities will be [n_subs, n_channels,
            int((n_ts-time_win)/time_step)+1, int((n_ts-time_win)/time_step)+1]
            If sub_opt=1 and chl_opt=0, the shape of CTSimilarities will be [n_subs, int((n_ts-time_win)/time_step)+1,
            int((n_ts-time_win)/time_step)+1]
            If sub_opt=0 and chl_opt=1, the shape of CTSimilarities will be [n_channels, int((n_ts-time_win)/time_step)
            +1, int((n_ts-time_win)/time_step)+1]
            If sub_opt=0 and chl_opt=0, the shape of CTSimilarities will be [int((n_ts-time_win)/time_step)+1,
            int((n_ts-time_win)/time_step)+1]
    """

    n_cons, n_subs, n_chls, n_ts = np.shape(data1)

    nts = int((n_ts - time_win) / time_step) + 1

    # chl_opt=0
    if chl_opt == 0:

        data1 = np.average(data1, axis=2)
        data2 = np.average(data2, axis=2)
        # shape : [n_subs, n_ts]

        newdata1 = np.zeros([n_subs, nts, time_win], dtype=np.float)
        newdata2 = np.zeros([n_subs, nts, time_win], dtype=np.float)

        for sub in range(n_subs):
            for t in range(nts):
                newdata1[sub, t] = data1[sub, t*time_step:t*time_step+time_win]
                newdata2[sub, t] = data2[sub, t*time_step:t*time_step+time_win]

        CTSimilarities = np.zeros([n_subs, nts, nts, 2], dtype=np.float)

        for sub in range(n_subs):
            for t1 in range(nts):
                for t2 in range(nts):

                    if method == 'spearman':
                        CTSimilarities[sub, t1, t2] = spearmanr(newdata1[sub, t1], newdata2[sub, t2])
                    if method == 'pearson':
                        CTSimilarities[sub, t1, t2] = pearsonr(newdata1[sub, t1], newdata2[sub, t2])
                    if method == 'kendall':
                        CTSimilarities[sub, t1, t2] = kendalltau(newdata1[sub, t1], newdata2[sub, t2])
                    if method == 'similarity':
                        V1 = np.mat(newdata1[sub, t1])
                        V2 = np.mat(newdata2[sub, t2])
                        num = float(V1 * V2.T)
                        denom = np.linalg.norm(V1) * np.linalg.norm(V2)
                        cos = num / denom
                        CTSimilarities[sub, t1, t2, 0] = 0.5 + 0.5 * cos
                    if method == 'distance':
                        CTSimilarities[sub, t1, t2, 0] = np.linalg.norm(newdata1[sub, t1] - newdata2[sub, t2])

        if sub_opt == 0:

            CTSimilarities = np.average(CTSimilarities, axis=0)

            if method == 'spearman' or method == 'pearson' or method == 'kendall':
                return CTSimilarities

            if method == 'similarity' or method == 'distance':
                return CTSimilarities[:, :, 0]

        if sub_opt == 1:

            if method == 'spearman' or method == 'pearson' or method == 'kendall':
                return CTSimilarities

            if method == 'similarity' or method == 'distance':
                return CTSimilarities[:, :, :, 0]

    if chl_opt == 1:

        newdata1 = np.zeros([n_subs, n_chls, nts, time_win], dtype=np.float)
        newdata2 = np.zeros([n_subs, n_chls, nts, time_win], dtype=np.float)

        for sub in range(n_subs):
            for chl in range(n_chls):
                for t in range(nts):
                    newdata1[sub, chl, t] = data1[sub, chl, t * time_step:t * time_step + time_win]
                    newdata2[sub, chl, t] = data2[sub, chl, t * time_step:t * time_step + time_win]

        CTSimilarities = np.zeros([n_subs, n_chls, nts, nts, 2], dtype=np.float)

        for sub in range(n_subs):
            for chl in range(n_chls):
                for t1 in range(nts):
                    for t2 in range(nts):

                        if method == 'spearman':
                            CTSimilarities[sub, chl, t1, t2] = spearmanr(newdata1[sub, t1], newdata2[sub, t2])
                        if method == 'pearson':
                            CTSimilarities[sub, chl, t1, t2] = pearsonr(newdata1[sub, t1], newdata2[sub, t2])
                        if method == 'kendall':
                            CTSimilarities[sub, chl, t1, t2] = kendalltau(newdata1[sub, t1], newdata2[sub, t2])
                        if method == 'similarity':
                            V1 = np.mat(newdata1[sub, chl, t1])
                            V2 = np.mat(newdata2[sub, chl, t2])
                            num = float(V1 * V2.T)
                            denom = np.linalg.norm(V1) * np.linalg.norm(V2)
                            cos = num / denom
                            CTSimilarities[sub, t1, t2, 0] = 0.5 + 0.5 * cos
                        if method == 'distance':
                            CTSimilarities[sub, t1, t2, 0] = np.linalg.norm(newdata1[sub, chl, t1] - newdata2[sub, chl, t2])

        if sub_opt == 0:

            CTSimilarities = np.average(CTSimilarities, axis=0)

            if method == 'spearman' or method == 'pearson' or method == 'kendall':
                return CTSimilarities

            if method == 'similarity' or method == 'distance':
                return CTSimilarities[:, :, :, 0]

        if sub_opt == 1:

            if method == 'spearman' or method == 'pearson' or method == 'kendall':
                return CTSimilarities

            if method == 'similarity' or method == 'distance':
                return CTSimilarities[:, :, :, :, 0]






# -*- coding: utf-8

"""
@File       :   ctrdm.py
@Author     :   Zitong Lu
@Contact    :   zitonglu1996@gmail.com
@License    :   MIT License
"""

' a module for plotting the Cross-Temporal Representational Dissimilarity Matrix (CTRDM) '

import numpy as np
import matplotlib.pyplot as plt


' a function for plotting the Cross-Temporal RDM '

def ctrdm_plot(CTRDM, percentile=False, lim=[0, 2], conditions=None, con_fontsize=12, cmap=None):

    """
    Plot the Cross-Temporal RDM

    Parameters
    ----------
    CTRDM : array or list [n_cons, n_cons]
        A Cross-Temporal RDM.
    lim : array or list [min, max]. Default is [0, 1].
        The corrs view lims.
    percentile : bool True or False. Default is False.
        Rescale the values in RDM or not by displaying the percentile.
    conditions : string-array or string-list. Default is None.
        The labels of the conditions for plotting.
        conditions should contain n_cons strings, If conditions=None, the labels of conditions will be invisible.
    con_fontsize : int or float. Default is 12.
        The fontsize of the labels of the conditions for plotting.
    cmap : matplotlib colormap. Default is None.
        The colormap for RDM.
        If cmap=None, the ccolormap will be 'jet'.

    Notes
    -----
    Only when percentile=False, lim works.
    """

    # get the number of conditions
    cons = CTRDM.shape[0]

    # if cons=2, the RDM cannot be plotted.
    if cons == 2:
        print("The shape of CTRDM cannot be 2*2. Here PyCTRSA cannot plot this CTRDM.")

        return None

    # determine if it's a square
    a, b = np.shape(CTRDM)
    if a != b:
        return None

    if percentile == True:

        v = np.zeros([cons*cons, 2], dtype=np.float)
        for i in range(cons):
            for j in range(cons):
                v[i*cons+j, 0] = CTRDM[i, j]

        index = np.argsort(v[:, 0])
        m = 0
        for i in range(cons*cons):
            if i > 0:
                if v[index[i], 0] > v[index[i - 1], 0]:
                    m = m + 1
                v[index[i], 1] = m

        v[:, 0] = v[:, 1] * 100 / m

        for i in range(cons):
            for j in range(cons):
                CTRDM[i, j] = v[i * cons + j, 0]

        if cmap == None:
            plt.imshow(CTRDM, extent=(0, 1, 0, 1), cmap=plt.cm.jet, clim=(0, 100))
        else:
            plt.imshow(CTRDM, extent=(0, 1, 0, 1), cmap=cmap, clim=(0, 100))

        # plt.axis("off")
        cb = plt.colorbar()
        cb.ax.tick_params(labelsize=16)
        font = {'size': 18}
        cb.set_label("Dissimilarity (percentile)", fontdict=font)

    else:

        # plot the RDM
        min = lim[0]
        max = lim[1]
        if cmap == None:
            plt.imshow(CTRDM, extent=(0, 1, 0, 1), cmap=plt.cm.jet, clim=(min, max))
        else:
            plt.imshow(CTRDM, extent=(0, 1, 0, 1), cmap=cmap, clim=(min, max))

        # plt.axis("off")
        cb = plt.colorbar()
        cb.ax.tick_params(labelsize=16)
        font = {'size': 18}
        cb.set_label("Dissimilarity", fontdict=font)

    if conditions != None:
        step = float(1 / cons)
        x = np.arange(0.5 * step, 1 + 0.5 * step, step)
        y = np.arange(1 - 0.5 * step, -0.5 * step, -step)
        plt.xticks(x, conditions, fontsize=con_fontsize, rotation=30, ha="right")
        plt.yticks(y, conditions, fontsize=con_fontsize)
    else:
        plt.axis("off")

    plt.xlabel("Time 1", fontsize=18)
    plt.ylabel("Time 2", fontsize=18)

    plt.show()